import tarfile
from .bcast import BroadCastCliSocket, BroadCastServSocket, BCAST_MAGIC
import socket
from os.path import basename
import glob

def mkpin(length=4):
  from random import choices
  from string import digits
  return ''.join(choices(digits, k=length))

def isurl(s):
  return s.startswith(('http://','https://'))

def main(argv):
  srv_addr, pin = None, ''
  if len(argv) > 1:
    if argv[1] in ('-up', '--userpin') and len(argv)>4:
      pin=argv[2]
      print('user PIN :', pin)
      argv = [argv[0]] + argv[3:]
    elif argv[1] in ('-p', '--pin'):
      pin=mkpin()
      print('generated PIN :', pin)
      argv = [argv[0]] + argv[2:]
    elif argv[1] in ('-r', '--recv', '--receiver'):
      srv_addr = (argv[2],)

  try:
    with BroadCastCliSocket(magic='sendfile'+pin) as s:
      while srv_addr is None:
        try:
          srv_addr = s.discovery()
        except socket.timeout:
          pass
    print('sending to :', (srv_addr[0], 18902))
    while True:
      try:
        with socket.create_connection((srv_addr[0], 18902), timeout=1) as s:
          with tarfile.open(fileobj=s.makefile('wb', buffering=0), mode='w|gz') as t:
            for a in argv[1:]:
              isMessage = isurl(a)
              if not isMessage:
                files = list(glob.iglob(a))
                isMessage |= 0 == len(files)
              if isMessage:
                t.addfile(tarfile.TarInfo(a))
              else:
                for f in files:
                  print('sending %s' %f)
                  t.add(f, arcname=basename(f))
            return 0
      except socket.timeout:
        pass
  except KeyboardInterrupt:
    print('user canceled sending')
    return 1
  
if __name__ == '__main__':
  from sys import argv
  main(argv)



def main(argv=None):
  if argv is None:
    from sys import argv
  def usage():
    print('usage : <send|s|recv|r> [arguments]')
    exit(1)
  if len(argv) == 1:
    usage()
  elif argv[1] in ('send', 's'):
    from .send import main as m
  elif argv[1] in ('recv', 'r'):
    from .recv import main as m
  else:
    usage()
  exit(m(argv[1:]))

if __name__ == '__main__':
  from sys import argv
  main(argv)



from time import sleep
from logging import info, debug, warn, critical, error
from socket import socket, timeout
import struct
import sys

BCAST_PORT = 31023 # 31022 for sync_play
BCAST_MAGIC = 'ch.ideas_bcast.py'

class BroadCastSocket(socket):
  def __init__(self, port=None, magic=None):
    from socket import AF_INET, SOCK_DGRAM
    super().__init__(AF_INET, SOCK_DGRAM)#, IPPROTO_UDP
    super().setblocking(True)
    super().settimeout(1)
    self.bcast_port = port or BCAST_PORT
    self.magic = (BCAST_MAGIC + (('_'+magic) if magic else '')).encode()
  
  def set_iface(self, iface):
    from sys import platform
    if platform == 'win32': return
    from socket import SOL_SOCKET, SO_BINDTODEVICE
    super().setsockopt(SOL_SOCKET, SO_BINDTODEVICE, iface.encode())

  def wait(self, num=0):
    pass

# Receiving side is Server
class BroadCastServSocket(BroadCastSocket):
  def __init__(self, port=None, magic=None):
    super().__init__(port, magic)
    from socket import AF_INET, SOL_SOCKET, SO_BROADCAST
    super().bind(('', 0))
    super().setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
    from sys import platform
    if platform == 'win32':
      from socket import gethostbyname_ex, gethostname
      self.bcast_addrs = [ip[:ip.rindex('.')+1]+'255' for ip in gethostbyname_ex(gethostname())[2] if not ip.startswith('127.')]
    #  from socket import inet_aton, getaddrinfo, gethostname
    #  addrs = getaddrinfo(gethostname(), 0, family=AF_INET)
    #  #ip = max(addrs, key=lambda v:int.from_bytes(inet_aton(v[-1][0]), 'big'))[-1][0]
    #  self.bcast_addrs = [v[:v.rindex('.')+1]+'255' for v in list(map(lambda v:v[-1][0], addrs))]
    #  #self.bcast_addr = '255.255.255.255'
    else:
      self.bcast_addrs = ['<broadcast>']
    #print('broadcasting to', self.bcast_addrs)

  def send(self, data):
    #info('server sending : %s', data)
    try:
      #super().sendto(data, (self.bcast_addr, self.bcast_port))
      for addr in self.bcast_addrs:
        #print('server sending : %s to %s:%s', data, addr, self.bcast_port)
        super().sendto(data, (addr, self.bcast_port))
    except Exception as e:
      error('socket send error with : %s', e)

  def announce(self):
    self.send(self.magic)

  def announce_loop(self):
    try:
      while True:
        try:
          self.announce()
        except timeout:
          pass
        sleep(1)
    except KeyboardInterrupt:
      print('finishing announce loop')

# Sending side is Client
class BroadCastCliSocket(BroadCastSocket):
  def __init__(self, port=None, magic=None):
    super().__init__(port, magic)
    from socket import SOL_SOCKET, SO_REUSEADDR
    super().setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    self.serv_addr = []
    super().bind(('', self.bcast_port))

  def discovery(self):
    try:
      data, addr = super().recvfrom(len(self.magic)+2)
      #print('discovered message :', data)
      #print('discovered from :', addr)
      return None if data[:len(self.magic)] != self.magic else addr
    except timeout:
      return None
    #if not addr in self.serv_addr:
    #  self.serv_addr.append(addr)
    #  print('added to server address :', addr)
  
  def send(self, data):
    if self.serv_addr is None: return
    super().sendto(data, self.serv_addr)

#def all_interfaces():
#  from array import array
#  from fcntl import ioctl
#  import socket
#  max_possible = 128 # arbitrary. raise if needed.
#  obytes = max_possible * 32
#  deb = b'\0'
#  s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
#  names = array('B', deb * obytes)
#  outbytes = struct.unpack('iL', ioctl(
#        s.fileno(),
#        0x8912,  # SIOCGIFCONF
#        struct.pack('iL', obytes, names.buffer_info()[0])
#        ))[0]
#
#  namestr = names.tostring()
#
#  lst = []
#  for i in range(0, outbytes, 40):
#    name = namestr[ i: i+16 ].split( deb, 1)[0]
#    name = name.decode()
#    #iface_name = namestr[ i : i+16 ].split( deb, 1 )[0]
#    ip = namestr[i+20:i+24]
#    lst.append((name, ip))
#    return lst

def test(is_serv):
  sock = (BroadCastServSocket if is_serv else BroadCastCliSocket)()
  try:
    if is_serv:
      sock.announce_loop()
    else:
      while not sock.discovery():
        pass
  except KeyboardInterrupt:
    print('test is canceled')
    sock.close()

if __name__ == '__main__':
  from sys import argv
  is_serv = len(argv)>1 and argv[1]=='s'
  test(is_serv)#, '' if len(argv)<3 else argv[2]) #argv[2] if len(argv)>2 and argv[2] else '')



import tarfile
from .bcast import BroadCastServSocket,BroadCastCliSocket, BCAST_MAGIC
import socket
import timeit
from threading import Thread

def openurl(url):
  import webbrowser
  try:
    if webbrowser.get():
      webbrowser.open(url)
  except webbrowser.Error:
    from shutil import which
    if which('termux-open-url'):
      from os import system
      system('termux-open-url '+url)

def extract_all(tar):
  from .send import isurl
  #mems = tar.getmembers()
  #print('length of tar members : ', len(mems))
  for m in tar:
    if m.size == 0:
      if isurl(m.name):
        openurl(m.name)
      else: # simple message
        print(m.name)
      continue
    extract(m, tar)
  
def extract(f, tar):
  print('extracting %s (%d bytes)'%(f.name,f.size))
  timeit.gc.disable() 
  t0=timeit.default_timer()
  tar.extract(f, filter='data')
  #read = tar.extractfile(f)
  t0=timeit.default_timer()-t0
  timeit.gc.enable()
  print(f'extracting {f.name} done ({f.size} bytes, {t0} seconds, {f.size/t0/10**6} MB/s)')
  
def main(argv):
  if len(argv)>1 and argv[1]:
    if argv[1] in ('n', 'new', 'g', 'gen'):
      from .send import mkpin
      pin = mkpin()
      print('generated PIN :', pin)
    else: pin = argv[1]
  else: pin = ''
    
  # receiving socket
  sock = None
  thread = None
  with socket.socket() as s:
    s.bind(('', 18902))
    s.setblocking(True)
    s.settimeout(1)
    s.listen()
    try:
      while True:
        with BroadCastServSocket(magic='sendfile'+pin) as sa:
          print('announcing')
          sa.announce()
        try:
          c, a = s.accept()
          print('receiving from : ', a[0]) # ip address
          fo = c.makefile('rb', buffering=0)
          tar = tarfile.open(fileobj=fo, mode='r|gz')
          
          thread = Thread(target=extract_all, args=[tar])
          thread.start()
          sock = sa
          break 
        except socket.timeout:
          continue
    except KeyboardInterrupt:
      s.close()
      print('user canceled receiving')
      return 1
  if thread.is_alive():
    thread.join()
    

if __name__ == '__main__':
  from sys import argv
  main(argv)
